@@ -21,14 +21,17 @@
 #ifndef BUILTINSTEST_H
 #define BUILTINSTEST_H
 
-#include <QtTest>
-#include <QtCore/QObject>
+#include <QtTest/QTest>
+#include <QtCore/QFileInfo>
+#include <QtCore/QDebug>
 
 #include "template.h"
 #include "engine.h"
 #include "context.h"
 #include "filterexpression.h"
 #include "grantlee_paths.h"
+#include "coverageobject.h"
+#include <metaenumvariable_p.h>
 
 typedef QHash<QString, QVariant> Dict;
 
@@ -184,13 +187,15 @@ public:
   }
 };
 
-class TestBuiltinSyntax : public QObject
+class TestBuiltinSyntax : public CoverageObject
 {
   Q_OBJECT
 
 private Q_SLOTS:
   void initTestCase();
 
+  void testObjects();
+
   void testBasicSyntax_data();
   void testBasicSyntax() {
     doTest();
@@ -262,12 +267,46 @@ private:
 
 };
 
+void TestBuiltinSyntax::testObjects()
+{
+  Context c1, c2;
+  c1 = c1;
+  c1 = c2;
+  Context c3(c1);
+  Q_UNUSED(c3);
+
+  FilterExpression f1, f2;
+  f1 = f1;
+  f1 = f2;
+  FilterExpression f3(f1);
+  Q_UNUSED(f3);
+
+  Variable v1;
+  v1 = v1;
+  v1 = f1.variable();
+  Variable v3(v1);
+  Q_UNUSED(v3);
+  QVERIFY(!v1.isTrue(&c1));
+  QVERIFY(!v1.isLocalized());
+
+  c1.setMutating(true);
+  QVERIFY(c1.isMutating());
+
+  SafeString s1, s2;
+  s1 = s1;
+  s2 = s1;
+  SafeString s3(s1);
+  Q_UNUSED(s3);
+
+  QMetaType::construct(qMetaTypeId<MetaEnumVariable>(), 0);
+}
+
 void TestBuiltinSyntax::initTestCase()
 {
   m_engine = getEngine();
-
   m_loader = InMemoryTemplateLoader::Ptr( new InMemoryTemplateLoader() );
   m_engine->addTemplateLoader( m_loader );
+  QVERIFY( m_engine->templateLoaders().contains( m_loader ) );
 }
 
 Engine* TestBuiltinSyntax::getEngine()
@@ -318,6 +357,9 @@ void TestBuiltinSyntax::testBasicSyntax_data()
 
   Dict dict;
 
+  QTest::newRow( "basic-syntax00" ) << QString() << dict << QString() << NoError;
+
+
   // Plain text should go through the template parser untouched
   QTest::newRow( "basic-syntax01" ) << QString::fromLatin1( "something cool" ) << dict << QString::fromLatin1( "something cool" ) << NoError;
 
@@ -484,6 +526,14 @@ void TestBuiltinSyntax::testBasicSyntax_data()
   dict.insert( QLatin1String( "abc" ), QLatin1String( "def" ) );
 
   QTest::newRow( "basic-syntax35" ) << QString::fromLatin1( "{{ abc._something }} {{ abc._something|upper }}" ) << dict << QString() << TagSyntaxError;
+
+  QTest::newRow( "basic-syntax36" ) << "{{ \"fred }}" << dict << QString() << TagSyntaxError;
+  QTest::newRow( "basic-syntax37" ) << "{{ \'fred }}" << dict << QString() << TagSyntaxError;
+  QTest::newRow( "basic-syntax38" ) << "{{ \"fred\' }}" << dict << QString() << TagSyntaxError;
+  QTest::newRow( "basic-syntax39" ) << "{{ \'fred\" }}" << dict << QString() << TagSyntaxError;
+  QTest::newRow( "basic-syntax40" ) << "{{ _(\'fred }}" << dict << QString() << TagSyntaxError;
+  QTest::newRow( "basic-syntax41" ) << "{{ abc|removetags:_(\'fred }}" << dict << QString() << TagSyntaxError;
+
 }
 
 void TestBuiltinSyntax::testEnums_data()
@@ -554,6 +604,8 @@ void TestBuiltinSyntax::testEnums_data()
                                       "{% endwith %}" )
                                    << dict << QString::fromLatin1( "2,4,8,Voter,Consumer,Citizen,,SomeClass" ) << NoError;
 
+  QTest::newRow( "class-enums36" ) << QString::fromLatin1( "{% ifequal var.Employee 2 %}{% endifequal %}" ) << dict << QString() << NoError;
+
   dict.insert( QLatin1String( "var" ), QVariant::fromValue( otherClass ) );
 
   QTest::newRow( "enums-loops01" ) << QString::fromLatin1( "{% for enum in var.Animals %}{% ifequal enum var.Tigers %}"
@@ -587,6 +639,7 @@ void TestBuiltinSyntax::testEnums_data()
   QTest::newRow( "qt-enums04" ) << QString::fromLatin1( "{{ Qt.AlignRight.value }}" ) << dict << QString::fromLatin1( "2" ) << NoError;
   QTest::newRow( "qt-enums05" ) << QString::fromLatin1( "{{ Qt.AlignRight.key }}" ) << dict << QString::fromLatin1( "AlignRight" ) << NoError;
   QTest::newRow( "qt-enums06" ) << QString::fromLatin1( "{{ Qt.Alignment.2.key }}" ) << dict << QString::fromLatin1( "AlignRight" ) << NoError;
+  QTest::newRow( "qt-enums06" ) << QString::fromLatin1( "{{ Qt.DoesNotExist }}" ) << dict << QString() << NoError;
 }
 
 void TestBuiltinSyntax::testListIndex_data()
@@ -674,7 +727,7 @@ void TestBuiltinSyntax::testFilterSyntax_data()
   // Raise TemplateSyntaxError for invalid block tags
   QTest::newRow( "filter-syntax07" ) << QString::fromLatin1( "{% nothing_to_see_here %}" ) << dict << QString() << InvalidBlockTagError;
   // Raise TemplateSyntaxError for empty block tags
-  QTest::newRow( "filter-syntax08" ) << QString::fromLatin1( "{% %}" ) << dict << QString() << InvalidBlockTagError;
+  QTest::newRow( "filter-syntax08" ) << QString::fromLatin1( "{% %}" ) << dict << QString() << EmptyBlockTagError;
 
   // Chained filters, with an argument to the first one
   dict.insert( QLatin1String( "var" ), QLatin1String( "<b><i>Yes</i></b>" ) );
@@ -725,6 +778,9 @@ void TestBuiltinSyntax::testFilterSyntax_data()
   dict.clear();
   QTest::newRow( "filter-syntax20" ) << "{{ \"\"|default_if_none:\"was none\" }}" << dict << QString() << NoError;
 
+
+  QTest::newRow( "filter-syntax21" ) << "{{ \"\"|default_if_none:|truncatewords }}" << dict << QString() << EmptyVariableError;
+
 }
 
 void TestBuiltinSyntax::testCommentSyntax_data()
@@ -791,9 +847,9 @@ void TestBuiltinSyntax::testEscaping_data()
   dict.insert( QLatin1String( "varList" ), QVariantList() << QString::fromLatin1( "Tom" ) << QString::fromLatin1( "Dick" ) << QString::fromLatin1( "Harry" ) );
   QTest::newRow( "escape05" ) << "{{ varList|join:\" & \" }}" << dict << QString::fromLatin1( "Tom & Dick & Harry" ) << NoError;
 
-  // Same with variable args.
+  // Unlike variable args.
   dict.insert( QLatin1String( "amp" ), QLatin1String( " & " ) );
-  QTest::newRow( "escape06" ) << QString::fromLatin1( "{{ varList|join:amp }}" ) << dict << QString::fromLatin1( "Tom & Dick & Harry" ) << NoError;
+  QTest::newRow( "escape06" ) << QString::fromLatin1( "{{ varList|join:amp }}" ) << dict << QString::fromLatin1( "Tom &amp; Dick &amp; Harry" ) << NoError;
 
   // Literal strings are safe.
   QTest::newRow( "escape07" ) << "{{ \"this & that\" }}" << dict << QString::fromLatin1( "this & that" ) << NoError;
@@ -1092,6 +1148,8 @@ void TestBuiltinSyntax::testTypeAccessors_data()
   dict.insert( QLatin1String( "str3" ), QLatin1String( "my string" ) );
   dict.insert( QLatin1String( "str4" ), QLatin1String( "my String" ) );
   dict.insert( QLatin1String( "str5" ), QLatin1String( "My string" ) );
+  dict.insert( QLatin1String( "str6" ), QLatin1String( "123" ) );
+  dict.insert( QLatin1String( "str7" ), QString() );
 
   QTest::newRow( "type-accessors-string34" ) << QString::fromLatin1( "{{ str1.title }}" ) << dict << QString::fromLatin1( "My String" ) << NoError;
   QTest::newRow( "type-accessors-string35" ) << QString::fromLatin1( "{{ str2.title }}" ) << dict << QString::fromLatin1( "Mystring" ) << NoError;
@@ -1102,6 +1160,11 @@ void TestBuiltinSyntax::testTypeAccessors_data()
   QTest::newRow( "type-accessors-string39" ) << QString::fromLatin1( "{{ str1.upper }}" ) << dict << QString::fromLatin1( "MY STRING" ) << NoError;
   QTest::newRow( "type-accessors-string40" ) << QString::fromLatin1( "{{ str2.upper }}" ) << dict << QString::fromLatin1( "MYSTRING" ) << NoError;
   QTest::newRow( "type-accessors-string41" ) << QString::fromLatin1( "{{ str3.upper }}" ) << dict << QString::fromLatin1( "MY STRING" ) << NoError;
+  QTest::newRow( "type-accessors-string42" ) << QString::fromLatin1( "{{ str3.dne }}" ) << dict << QString() << NoError;
+  QTest::newRow( "type-accessors-string43" ) << QString::fromLatin1( "{{ str2.isalpha }}" ) << dict << QString::fromLatin1( "True" ) << NoError;
+  QTest::newRow( "type-accessors-string44" ) << QString::fromLatin1( "{{ str3.isalpha }}" ) << dict << QString::fromLatin1( "False" ) << NoError;
+  QTest::newRow( "type-accessors-string45" ) << QString::fromLatin1( "{{ str6.isalpha }}" ) << dict << QString::fromLatin1( "False" ) << NoError;
+  QTest::newRow( "type-accessors-string46" ) << QString::fromLatin1( "{{ str7.isalpha }}" ) << dict << QString::fromLatin1( "False" ) << NoError;
 
   dict.clear();
 
@@ -1283,7 +1346,9 @@ void TestBuiltinSyntax::testInsignificantWhitespace()
 
   Context context( dict );
 
+  QVERIFY(!m_engine->smartTrimEnabled());
   m_engine->setSmartTrimEnabled(true);
+  QVERIFY(m_engine->smartTrimEnabled());
 
   {
     Template t = m_engine->newTemplate( input, QLatin1String( QTest::currentDataTag() ) );
@@ -1527,6 +1592,11 @@ void TestBuiltinSyntax::testInsignificantWhitespace_data()
                         << QString::fromLatin1("\n{;\n")
                         << QString::fromLatin1("\n{;\n");
 
+  QTest::newRow( "insignificant-whitespace44" )
+      << QString::fromLatin1("\n{{ foo }} " ) << dict
+                        << QString()
+                        << QString::fromLatin1("\n ");
+
 }
 
 QTEST_MAIN( TestBuiltinSyntax )
